/*
<:copyright-broadcom 
 
 Copyright (c) 2006 Broadcom Corporation 
 All Rights Reserved 
 No portions of this material may be reproduced in any form without the 
 written permission of: 
          Broadcom Corporation 
          16215 Alton Parkway 
          Irvine, California 92619 
 All information contained in this document is Broadcom Corporation 
 company private, proprietary, and trade secret. 
 
:>
*/
/***************************************************************************
 * File Name  : atmbondingdrv.h
 *
 * Description: This file contains the definitions and structures for the
 *              Linux IOCTL interface that used between the user mode ATM
 *              BONDING library and the kernel ATM Bonding Driver.
 *
 * Updates    : 02/22/2006 Created.
 ***************************************************************************/

#if !defined(_ATMBDRV_H_)
#define _ATMBDRV_H_

#if defined(__cplusplus)
extern "C" {
#endif

#ifdef BCM_ATM_BONDING
/* Incldes. */
#include <bcmatmapi.h>
#include <bcmatmbapi.h>

/* Defines. */
#define ATMBDRV_MAJOR            226 /* arbitrary unused value */

#define ATMBIOCTL_INITIALIZE \
    _IOWR(ATMBDRV_MAJOR, 0, ATMB_INITIALIZE)
#define ATMBIOCTL_UNINITIALIZE \
    _IOR(ATMBDRV_MAJOR, 1, ATMB_STATUS_ONLY)
#define ATMBIOCTL_SET_INTERFACE_CFG \
    _IOWR(ATMBDRV_MAJOR, 2, ATMB_INTERFACE_CFG)
#define ATMBIOCTL_SET_VCC_CFG \
    _IOWR(ATMBDRV_MAJOR, 3, ATMB_VCC_CFG)
#define ATMBIOCTL_GET_STATISTICS \
    _IOWR(ATMBDRV_MAJOR, 4, ATMB_STATISTICS)
#define ATMBIOCTL_GET_MGMT_STATISTICS \
    _IOWR(ATMBDRV_MAJOR, 5, ATMB_MGMT_STATISTICS)
#define ATMBIOCTL_SET_INTERFACE_LINK_INFO \
    _IOWR(ATMBDRV_MAJOR, 6, ATMB_INTERFACE_LINK_INFO)


#define MAX_ATMBDRV_IOCTL_COMMANDS   7

/* Typedefs. */
typedef struct
{
    BCMATMB_STATUS baStatus ;
} ATMB_STATUS_ONLY, *PATMB_STATUS_ONLY ;

typedef struct
{   
    AtmBRcvBufParams  *pInit ;
    BCMATMB_STATUS    baStatus ;
} ATMB_INITIALIZE, *PATMB_INITIALIZE ;

typedef struct
{
    UINT32                  ulInterfaceId ;
    ATM_INTERFACE_LINK_INFO InterfaceCfg ;
    BCMATMB_STATUS          baStatus ;
} ATMB_INTERFACE_LINK_INFO, *PATMB_INTERFACE_LINK_INFO;

typedef struct _AtmBBdQ {
   ATM_DMA_BD      **ppStart ;        // address of queue in SDRAM on 16 byte boundary
   volatile UINT32 *pulHeadIdx ;      // addr/data of ATM Processor register head index
   volatile UINT32 *pulTailIdx ;      // addr/data of ATM Processor register tail index
   UINT32          ulLength ;         // length of queue

   /* Following are necessary fpr mgmt module(s) not the data path modules. */
   union {
      ATM_DMA_BD   **ppStartBase ;    // address of queue in SDRAM
      UINT32       resv ;
   } uAddr ;
   volatile UINT32 ulRegisterBase ; // SAR Register address.
} AtmBBdQ ;

typedef struct _AtmBAtmApiIf {
   AtmBBdQ  freePktQ ;
   AtmBBdQ  rxPktQ ;
} AtmBAtmApiIf ;

BCMATMB_STATUS BcmAtmb_SetAtmBIf (AtmBAtmApiIf **ppIf) ;

#if defined(__cplusplus)
}
#endif

#endif /* BCM_ATM_BONDING */

#endif // _ATMBDRV_H_
